/*
 * Java 1.7
 *
 * Copyright 2017-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.security.util;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * This InputStream implementation is used to limit the reading operations to a defined length of a DER encoded stream.
 * The input stream passed as the parameter of the constructor MUST be positioned right be before the Tag byte of a DER
 * value. This {@link DerValueInputStream} can be considered as a limited, narrowed, input stream. If this
 * {@link DerValueInputStream} can only be read for the length of the DER value the {@link InputStream} passed as the
 * parameter of the constructor may contain more data.
 *
 * <p>
 *
 * In addition, a method is called after each read: {@link DerValueInputStream#read()} will automatically call
 * {@link DerValueInputStream#onDataRead(byte)}, {@link DerValueInputStream#read(byte[], int, int)} will automatically
 * call {@link DerValueInputStream#onDataRead(byte[], int, int)}, and {@link DerValueInputStream#skip(long)} performs
 * {@link DerValueInputStream#read()} operations. Finally {@link DerValueInputStream#onStreamEnd()} is call right before
 * returning the last byte of the DER value.
 * <p>
 * A classic usage of this class is a signed DER Value, as shown in the following code example. A hash is updated in
 * {@link DerValueInputStream#onDataRead(byte[], int, int)} and {@link DerValueInputStream#onDataRead(byte)} and the
 * signature is verified in {@link DerValueInputStream#onStreamEnd()}.
 *
 * <pre>
    final Signature signature;
    [...]
    DerFixedLengthInputStream filteredInputStream = new DerFixedLengthInputStream(signedCommand) {
        &#64;Override
        protected void onStreamEnd() throws IOException {
            validateSignature(signature);
        }

        &#64;Override
        protected void onDataRead(byte[] data, int off, int len) {
            try {
                 signature.update(data, off, len);
            } catch (SignatureException e) {
                 throw new InvalidSignatureException(e);
            }
        }

        &#64;Override
        protected void onDataRead(byte b) {
            try {
                 signature.update(b);
            } catch (SignatureException e) {
                 throw new InvalidSignatureException(e);
            }
        }
    };
 * </pre>
 */
public class DerValueInputStream extends FilterInputStream {
	private boolean streamEnded;
	/**
	 * Number of remaining byte of the stream before reaching its end.
	 */
	protected int remainingBytes;
	/**
	 * DER Tag of the stream, determined by the first byte of the stream.
	 */
	protected final int tag;

	/**
	 * Constructs a fixed length input stream of a DER encoded stream. The first bytes of the stream are automatically
	 * read in the constructor in order to determine the tag value as well as the length of the stream.
	 *
	 * @param in
	 *            the input stream to limit.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws DerFormatException
	 *             if an error occurs while parsing.
	 */
	public DerValueInputStream(InputStream in) throws IOException, DerFormatException {
		super(in);
		// this temporary non zero value is there to avoid triggering verification in read operation
		// as the signature verification is triggered when teh stream is entirely consumed, i.e. length reaches zero
		this.remainingBytes = Integer.MAX_VALUE;

		this.tag = this.read();
		// we must read through "self", as the read bytes are part of the signature.
		this.remainingBytes = DerInputStreamHelper.readDERLength(this);
	}

	@Override
	public void close() {
		// Do nothing
	}

	/**
	 * Called once the end of the stream is reached. This allows, for instance, to retrieve a hash or perform a
	 * signature verification.
	 *
	 * @throws IOException
	 *             This this method may read further in the input stream passed as parameter of the constructor, it may
	 *             throw {@link IOException} on I/O errors.
	 * @throws RuntimeException
	 *             As this method can be user-defined, it may throw only {@link IOException}, {@link Error} or
	 *             {@link RuntimeException}.
	 */
	protected void onStreamEnd() throws IOException, RuntimeException {
		// no-op
	}

	/**
	 * Called after a data read to apply a process on it, such as an update of hash.
	 *
	 * @param data
	 *            the array of bytes.
	 * @param off
	 *            the offset to start from in the array of bytes.
	 * @param len
	 *            the number of bytes to use, starting at offset.
	 * @throws RuntimeException
	 *             As this method can be user-defined, it may throw only {@link IOException}, {@link Error} or
	 *             {@link RuntimeException}.
	 */
	protected void onDataRead(byte[] data, int off, int len) throws RuntimeException {
		// no-op
	}

	/**
	 * Called after a data read to apply a process on it, such as an update of hash.
	 *
	 * @param b
	 *            the byte to use for the update.
	 * @throws RuntimeException
	 *             As this method can be user-defined, it may throw only {@link IOException}, {@link Error} or
	 *             {@link RuntimeException}.
	 */
	protected void onDataRead(byte b) throws RuntimeException {
		// no-op
	}

	@Override
	public int available() throws IOException {
		return Math.min(super.available(), this.remainingBytes);
	}

	/**
	 * If <code>len</code> is bigger than {@link #remainingBytes}, then <code>len</code> is set to
	 * {@link #remainingBytes}
	 */
	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		try {
			if (this.remainingBytes <= 0) {
				return -1;
			}

			len = Math.min(this.remainingBytes, len);

			int result = this.in.read(b, off, len);
			if (result != -1) {
				this.remainingBytes -= result;
				this.onDataRead(b, off, result);
			}

			return result;
		} finally {
			if (this.remainingBytes <= 0) {
				checkStreamEnd();
			}
		}
	}

	@Override
	public int read() throws IOException {
		try {
			if (this.remainingBytes <= 0) {
				return -1;
			}

			int ch = this.in.read();
			if (ch != -1) {
				this.remainingBytes--;
				this.onDataRead((byte) ch);
			}

			return ch;
		} finally {
			if (this.remainingBytes <= 0) {
				checkStreamEnd();
			}
		}
	}

	/**
	 * Checks if the end of the stream is reached. If so, triggers signature verification.
	 *
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws RuntimeException
	 *             Any {@link RuntimeException} could be thrown by this process.
	 */
	protected void checkStreamEnd() throws IOException, RuntimeException {
		if (!this.streamEnded) {
			this.streamEnded = true;
			this.onStreamEnd();
		}
	}

	/**
	 * Consumes, and drops, a given number of bytes. These bytes are consumed instead of just dropped in order to update
	 * the hash.
	 *
	 * @param n
	 *            the number of bytes to skip.
	 * @return the number of bytes actually skipped.
	 * @throws IOException
	 *             if an I/O error occurs.
	 */
	@Override
	public long skip(long n) throws IOException {
		long skippedBytes = 0;
		for (long i = 0; i < n; i++) {
			this.read();
			skippedBytes++;
		}
		return skippedBytes;
	}

	/**
	 * Returns the number of bytes that remains in the stream.
	 *
	 * @return the remaining number of bytes.
	 */
	public int remainingLength() {
		return this.remainingBytes;
	}

	/**
	 * Returns the tag of the stream, e.g. first byte read.
	 *
	 * @return the DER tag of the stream.
	 */
	public int getTag() {
		return this.tag;
	}
}
