/*
 * Java
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.wadapps.management.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import ej.observable.Observer;
import ej.wadapps.app.BackgroundService;
import ej.wadapps.management.BackgroundServicesList;

/**
 * Starts backgrounds as soon as they are registered in the managed background
 * list and stops them when they are unregistered.
 */
public class BackgroundsManager implements Observer {

	private final Collection<BackgroundService> runningBackgrounds;
	private BackgroundServicesList backgroundsList;
	private boolean started;

	/**
	 * Default constructor.
	 */
	public BackgroundsManager() {
		this.runningBackgrounds = new ArrayList<>();
	}

	/**
	 * Sets the backgrounds list to manage.
	 * <p>
	 * Starts all backgrounds registered in the background list.
	 *
	 * @param backgroundsList
	 *            the backgrounds list to set.
	 */
	public synchronized void setBackgroundsList(BackgroundServicesList backgroundsList) {
		internalStop();
		this.backgroundsList = backgroundsList;
		internalStart();
	}

	/**
	 * Starts the background manager.
	 * <p>
	 * Starts all backgrounds registered in the background list.
	 */
	public synchronized void start() {
		this.started = true;
		internalStart();
	}

	/**
	 * Stops the background manager.
	 * <p>
	 * Starts all started backgrounds.
	 */
	public synchronized void stop() {
		this.started = false;
		internalStop();
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void internalStart() {
		BackgroundServicesList backgroundsList = this.backgroundsList;
		if (this.started && backgroundsList != null) {
			backgroundsList.addObserver(this);
			BackgroundService[] backgrounds = backgroundsList.getBackgroundServices();
			for (BackgroundService background : backgrounds) {
				start(background);
			}
		}
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void internalStop() {
		BackgroundServicesList backgroundsList = this.backgroundsList;
		if (backgroundsList != null) {
			backgroundsList.deleteObserver(this);
			for (BackgroundService background : this.runningBackgrounds) {
				background.onStop();
			}
			this.runningBackgrounds.clear();
		}
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void start(BackgroundService background) {
		background.onStart();
		this.runningBackgrounds.add(background);
	}

	@Override
	public synchronized void update() {
		BackgroundService[] backgroundServices = this.backgroundsList.getBackgroundServices();
		// search for new backgrounds
		for (BackgroundService backgroundService : backgroundServices) {
			if (this.started && !this.runningBackgrounds.contains(backgroundService)) {
				start(backgroundService);
			}
		}
		// search for stale backgrounds
		Iterator<BackgroundService> iterator = this.runningBackgrounds.iterator();
		while (iterator.hasNext()) {
			BackgroundService runningBackgroundService = iterator.next();
			if (!ArrayUtils.contains(backgroundServices, runningBackgroundService)) {
				iterator.remove();
				runningBackgroundService.onStop();
			}
		}
	}

}