/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import ej.basedriver.Controller;
import ej.basedriver.ControllerListener;
import ej.ecom.DeviceManager;
import ej.ecom.RegistrationEvent;
import ej.ecom.RegistrationListener;
import ej.ecom.io.CommPort;

public abstract class AbstractDriverService implements DriverService, RegistrationListener<CommPort> {
	private final Map<CommPort, Controller> ports;
	private final ControllerListener controllerListener;

	/**
	 *
	 */
	public AbstractDriverService() {
		controllerListener = createControllerListener();
		ports = new HashMap<>();
	}

	/**
	 * Create the controllerListener.
	 * @return the controllerListener.
	 */
	protected ControllerListener createControllerListener() {
		return new EcomEventControllerListener();
	}

	@Override
	public void start() {
		DeviceManager.addRegistrationListener(this, CommPort.class);
		// list already registered ports
		Iterator<CommPort> it = DeviceManager.list(CommPort.class);
		while(it.hasNext()){
			CommPort p = it.next();
			tryToCreateController(p);
		}
	}

	@Override
	public void stop() {
		DeviceManager.removeRegistrationListener(this);
		for (Controller controller : ports.values()) {
			stopController(controller);
		}
	}

	@Override
	public void deviceRegistered(RegistrationEvent<CommPort> event) {
		tryToCreateController(event.getDevice());
	}

	private void tryToCreateController (CommPort port) {
		Controller controller = create(port);
		if (controller != null && controller.start(controllerListener)) {
			ports.put(port, controller);
		}
	}

	@Override
	public void deviceUnregistered(RegistrationEvent<CommPort> event) {
		CommPort port = event.getDevice();
		if (ports.containsKey(port)) {
			stopController(ports.remove(port));
		}
	}

	private void stopController (Controller controller) {
		if (controller.isStarted()) {
			controller.stop();
		}
	}

	protected abstract Controller create (CommPort port);
}
