/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.util;

/**
 * Utility class that implements a sequence of bits.
 */
public class BitArray {
	private static final int SIZE = 8;
	private final byte [] bytes;
	private final int length;

	/**
	 * Constructs a new {@code BitArray} by using the specified byte.
	 * @param b
	 * 			The byte to use.
	 */
	public BitArray (byte b) {
		this(new byte [] {b});
	}

	/**
	 *  Constructs a new {@code BitArray} by using the specified array of bytes.
	 * @param bytes
	 *				The bytes to use.
	 */
	public BitArray (byte [] bytes) {
		this(bytes, 0, bytes.length);
	}

	/**
	 * Constructs a new {@code BitArray} by using the specified subarray of bytes.
	 * @param bytes
	 * 				The bytes to use.
	 * @param offset
	 * 				The index of the first byte to use
	 * @param length
	 * 				The number of bytes to use
	 * @throws IndexOutOfBoundsException
	 *          If the {@code offset} and the {@code length} arguments index
	 *          characters outside the bounds of the {@code bytes} array.
	 */
	public BitArray (byte [] bytes, int offset, int length) {
		this.bytes = new byte [length];
		System.arraycopy(bytes, offset, this.bytes, 0, length);
		this.length = length * 8;
	}

	/**
	 * Initializes a newly created {@code BitArray} object so that it represents
	 * an bit sequence which all bits is not set.
	 * @param length
	 * 				The length of this bitarray.
	 */
	public BitArray (int length) {
		this.length = length;
		int q = length / SIZE;
		int r = length % SIZE;
		if (r > 0) {
			q++;
		}

		this.bytes = new byte [q];
	}

	/**
	 * Returns this sequence of bits into an array of bytes.
	 * @return the bit sequence into an array of bytes
	 */
	public byte [] getBytes () {
		byte [] bytes = new byte [this.bytes.length];
		System.arraycopy(this.bytes, 0, bytes, 0, this.bytes.length);
		return bytes;
	}

	/**
	 * Set the specified bit.
	 * @param position the position of the bit to set.
	 */
	public void setBit (int position) {
		checkBounds(position);
		int q = position / SIZE;
		int r = position % SIZE;
		bytes[q] = (byte) (bytes[q] | ((byte) (1 << r)));
	}

	/**
	 * Clear the specified bit.
	 * @param position the position of the bit to clear.
	 */
	public void clearBit (int position) {
		checkBounds(position);
		int q = position / SIZE;
		int r = position % SIZE;
		bytes[q] = (byte) (bytes[q] & ((byte) (0 << r)));
	}

	/**
	 * Tests if the specified bit is set or not.
	 * @param position the position of the bit to test.
	 * @return true if the specified bit is set, false otherwise.
	 */
	public boolean testBit (int position) {
		checkBounds(position);
		int q = position / SIZE;
		int r = position % SIZE;
		return (bytes[q] & ((byte) (1 << r))) != 0;
	}

	private void checkBounds (int position) {
		if (position < 0 || position >= length) {
			throw new IndexOutOfBoundsException("Index < 0 or index >= " + length + '.');
		}
	}
}
