/*
 * Java
 *
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.components.dependencyinjection;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;

/**
 * A composite service loader contains a list a service loaders. It searches the requested services in its children
 * sequentially respecting the order they are added.
 */
public class CompositeServiceLoader implements ServiceLoader {

	private final DefaultServiceLoader defaultServiceLoader;
	private ServiceLoader[] serviceLoaders;

	/**
	 * Creates a new empty composite service loader.
	 */
	public CompositeServiceLoader() {
		this.serviceLoaders = new ServiceLoader[0];
		this.defaultServiceLoader = new DefaultServiceLoader();
	}

	/**
	 * Creates a new empty composite service loader with a default size.
	 *
	 * @param size
	 *            the expected number of children.
	 * @deprecated use {@link #CompositeServiceLoader()}
	 */
	@Deprecated
	public CompositeServiceLoader(int size) {
		this();
	}

	/**
	 * Adds a service loader at the end of the list a service loaders used by this composite service loader.
	 *
	 * @param serviceLoader
	 *            the service loader to add.
	 * @throws NullPointerException
	 *             if the given service loader is <code>null</code>.
	 */
	public void addServiceLoader(ServiceLoader serviceLoader) {
		if (serviceLoader == null) {
			throw new NullPointerException();
		}
		this.serviceLoaders = ArrayTools.add(this.serviceLoaders, serviceLoader);
	}

	@Override
	@Nullable
	public <T> T getService(Class<T> service) {
		for (ServiceLoader serviceLoader : this.serviceLoaders) {
			T serviceInstance = serviceLoader.getService(service);
			if (serviceInstance != null) {
				return serviceInstance;
			}
		}
		return this.defaultServiceLoader.getService(service);
	}

	@Override
	public <T> T getService(Class<T> service, Class<? extends T> defaultImplementation) {
		T serviceInstance = this.getService(service);
		if (serviceInstance == null) {
			serviceInstance = this.defaultServiceLoader.getService(service, defaultImplementation);
		}
		return serviceInstance;
	}

	@Override
	@Nullable
	public <T> T getService(Class<T> service, boolean singleton) {
		for (ServiceLoader serviceLoader : this.serviceLoaders) {
			T serviceInstance = serviceLoader.getService(service, singleton);
			if (serviceInstance != null) {
				return serviceInstance;
			}
		}
		return this.defaultServiceLoader.getService(service, singleton);
	}

	@Override
	public <T> T getService(Class<T> service, Class<? extends T> defaultImplementation, boolean singleton) {
		T serviceInstance = this.getService(service, singleton);
		if (serviceInstance == null) {
			serviceInstance = this.defaultServiceLoader.getService(service, defaultImplementation, singleton);
		}
		return serviceInstance;
	}

}
