/*
 * Java
 *
 * Copyright 2009-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.fp.widget;

import ej.fp.FrontPanel;
import ej.fp.Image;
import ej.fp.Widget.WidgetAttribute;
import ej.fp.Widget.WidgetDescription;

/**
 * This widget is an implementation of MicroUI widget display to be compatible with MicroEJ platform and chosen
 * graphical engine.
 * <p>
 * The filter image is used to define the display rendering area. Outside this area, the display buffer data is not
 * drawn. This image must have the same size (@see {@link #setWidth(int)} and {@link #setHeight(int)}) than the widget
 * itself. If not, an error is thrown in {@link #finalizeConfiguration()}.
 */
@SuppressWarnings("nls")
@WidgetDescription(attributes = { @WidgetAttribute(name = "label", isOptional = true), @WidgetAttribute(name = "x"),
		@WidgetAttribute(name = "y"), @WidgetAttribute(name = "width"), @WidgetAttribute(name = "height"),
		@WidgetAttribute(name = "displayWidth", isOptional = true),
		@WidgetAttribute(name = "displayHeight", isOptional = true),
		@WidgetAttribute(name = "initialColor", isOptional = true), @WidgetAttribute(name = "alpha", isOptional = true),
		@WidgetAttribute(name = "filter", isOptional = true),
		@WidgetAttribute(name = "extensionClass", isOptional = true) })
public class Display extends MicroUIDisplay {

	/**
	 * When the display is double buffered, the backBuffer represents the back buffer where drawings are done. At flush
	 * time, the backBuffer is copied on this display. When the display is simple buffered, the backBuffer and frame
	 * buffer are equals and all drawings are performed in this buffer.
	 */
	protected Image drawingBuffer;

	/**
	 * This buffer contains the application drawings after the call to {@link #flush()} in double buffer mode or during
	 * application rendering in simple buffer mode.
	 */
	protected Image frameBuffer;

	/**
	 * Buffer visible on front panel viewer: contains application drawings (see {@link #frameBuffer}) and some post
	 * transformations like mask reduction, backlight and contrast.
	 */
	protected Image visibleBuffer;

	// display characteristics
	private int initialColor; // 0 init
	private byte alpha;

	/**
	 * Creates the widget display
	 */
	public Display() {
		// fully opaque by default
		this.alpha = (byte) 0xff;
	}

	/**
	 * Sets the initial color of the display (color used on startup, just before first drawing). By default the initial
	 * color is 0x0 (black).
	 * <p>
	 * This is a RGB color (24 bits: 8-8-8). Alpha level is ignored.
	 * <p>
	 * This method should only be called by front panel parser.
	 *
	 * @param initialColor
	 *            the initial color to set.
	 */
	public void setInitialColor(int initialColor) {
		this.initialColor = initialColor | 0xff000000;
	}

	/**
	 * Sets the opacity level (alpha) of the display in order to obtain a translucent display. By default the display is
	 * fully opaque.
	 * <p>
	 * This method should only be called by front panel parser.
	 *
	 * @param alpha
	 *            the opacity level to set, 0x00 is fully transparent and 0xff fully opaque.
	 * @throws IllegalArgumentException
	 *             if the given argument is not between 0x0 and 0xff.
	 */
	public void setAlpha(int alpha) {
		if (alpha < 0x0 || alpha > 0xff) {
			throw new IllegalArgumentException("The opacity level must be a value between 0x0 and 0xff");
		}
		this.alpha = (byte) alpha;
	}

	@Override
	public void showYourself(boolean appearSwitchedOn) {
		getDrawingBuffer().fillRectangle(0, 0, getDisplayWidth(), getDisplayHeight(), convertDisplayColorToARGBColor(
				convertARGBColorToDisplayColor(appearSwitchedOn ? ~getInitialColor() : getInitialColor())));
		flush();
	}

	/**
	 * Called by the parser after filling all the fields defined in the xml. Used to complete the display
	 * initialization.
	 */
	@Override
	public void start() {

		super.start();

		// crop initial color to display capacities
		this.initialColor = convertDisplayColorToARGBColor(convertARGBColorToDisplayColor(this.initialColor));

		// create image returned by getCurrentSkin(); this image may be smaller or higher than display buffers
		this.visibleBuffer = newWidgetImage();

		// create image which will contains the drawings
		this.frameBuffer = newDisplayImage();

		// create image where performing drawings; it is frame buffer in simple buffer mode or a dedicated back buffer
		// in double buffer mode.
		this.drawingBuffer = isDoubleBuffered() ? newDisplayImage() : this.frameBuffer;

		// turn on backlight
		switchBacklight(true);
	}

	@Override
	public void dispose() {
		FrontPanel.getFrontPanel().disposeIfNotNull(this.frameBuffer, this.visibleBuffer);
		if (this.drawingBuffer != this.frameBuffer) {
			FrontPanel.getFrontPanel().disposeIfNotNull(this.drawingBuffer);
		}
		this.frameBuffer = null;
		this.visibleBuffer = null;
		this.drawingBuffer = null;
		super.dispose();
	}

	/**
	 * Gets the current displayed skin.
	 *
	 * @return the current displayed skin.
	 */
	@Override
	public synchronized Image getCurrentSkin() {

		// draw frame buffer data on visible buffer (scale up/down if necessary)
		Image resultImage = this.visibleBuffer;
		resultImage.drawImage(this.frameBuffer);

		// apply contrast and backlight
		drawContrast(resultImage);
		drawBacklight(resultImage);

		// apply transparency
		byte alpha = this.alpha;
		if (alpha != 0xff) {
			resultImage = resultImage.getTransparentImage(alpha);
		}

		// crop displayed area according mask
		resultImage.crop(getFilter());

		return resultImage;
	}

	@Override
	public Image getDrawingBuffer() {
		return this.drawingBuffer;
	}

	/**
	 * Gets the initial color of the display.
	 *
	 * @return the initial color.
	 */
	public int getInitialColor() {
		return this.initialColor;
	}

	/**
	 * Overrides default behavior: backlight is always available on a pixalated display.
	 */
	@Override
	public boolean hasBacklight() {
		return true;
	}

	@Override
	public void flush(int x, int y, int width, int height) {

		if (isDoubleBuffered()) {
			// draw back buffer data in frame buffer
			this.frameBuffer.drawImage(this.drawingBuffer, x, y, width, height, x, y, width, height);
		}

		// launch repaint
		if (this.alpha != 0xff) {
			// parent must repaint the display background
			getParent().repaint(getX(), getY(), getWidth(), getHeight());
		} else {
			// all display should be repaint because the realSize and the size
			// can be different
			repaint();
		}
	}

	/**
	 * Creates a new image with the same size than display (not widget size).
	 *
	 * @return an image as big as display.
	 */
	protected Image newDisplayImage() {
		return FrontPanel.getFrontPanel().newImage(getDisplayWidth(), getDisplayHeight(), this.initialColor, true);
	}

	/**
	 * Creates a new image with the same size than widget (not display size).
	 *
	 * @return an image as big as widget.
	 */
	protected Image newWidgetImage() {
		return FrontPanel.getFrontPanel().newImage(getWidth(), getHeight(), this.initialColor, true);
	}

	/**
	 * Transforms the skin data applying an algorithm to simulate the display contrast.
	 * <p>
	 * The default implementation performs nothing.
	 *
	 * @param imageSkin
	 *            the image where applying the algorithm.
	 */
	protected void drawContrast(Image imageSkin) {
		// not implemented
	}

	/**
	 * Transforms the skin data applying an algorithm to simulate the display backlight.
	 *
	 * @param imageSkin
	 *            the image where applying the algorithm.
	 */
	protected void drawBacklight(Image imageSkin) {
		if (hasBacklight()) {
			// get a transparency level according backlight
			int shadow = ((BACKLIGHT_MAX - getBacklight()) * 0xff / BACKLIGHT_MAX);
			imageSkin.fillTransparentRectangle(0, 0, imageSkin.getWidth(), imageSkin.getHeight(), shadow << 24);
		}
	}
}
