/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.font.filter;

import java.util.ArrayList;
import java.util.List;

/**
 * A composite filter delegates the filtering to other filters.
 * <p>
 * A composite filter accepts an element if all its children filters accept the element.
 *
 * @param <T>
 *            the type of element to check.
 */
public class CompositeFilter<T> implements Filter<T> {

	private final List<Filter<T>> fontFilters;

	/**
	 * Creates a composite filter.
	 */
	public CompositeFilter() {
		this.fontFilters = new ArrayList<Filter<T>>(2);
	}

	/**
	 * Adds a filter in this composite.
	 *
	 * @param filter
	 *            the filter to add.
	 * @throws NullPointerException
	 *             if the given filter is <code>null</code>.
	 */
	public void addFilter(Filter<T> filter) {
		if (filter == null) {
			throw new NullPointerException();
		}
		this.fontFilters.add(filter);
	}

	/**
	 * Removes a font filter in this composite.
	 *
	 * @param filter
	 *            the filter to remove.
	 */
	public void removeFilter(Filter<T> filter) {
		this.fontFilters.remove(filter);
	}

	@Override
	public boolean accept(T element) {
		for (Filter<T> filter : this.fontFilters) {
			if (!filter.accept(element)) {
				return false;
			}
		}
		// All filters accept the element.
		return true;
	}

}
