/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;

/**
 * A class selector selects from a class. That means that the class of this selector is available in the element
 * classes.
 * <p>
 * Equivalent to <code>.class</code> selector in CSS. Its specificity is (0,0,1,0).
 *
 * @see Element#hasClassSelector(String)
 * @see SpecificityHelper
 */
public class ClassSelector implements Selector {

	private final String classSelector;

	/**
	 * Creates a class selector.
	 *
	 * @param classSelector
	 *            the class.
	 */
	public ClassSelector(String classSelector) {
		this.classSelector = classSelector;
	}

	@Override
	public boolean fit(Element element) {
		// Check that the class of this selector is available in the given element.
		return element.hasClassSelector(this.classSelector);
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(0, 0, 1, 0);
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof ClassSelector) {
			ClassSelector other = (ClassSelector) obj;
			return this.classSelector.equals(other.classSelector);
		}
		return false;
	}

	@Override
	public int hashCode() {
		return this.classSelector.hashCode();
	}
}
