/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import ej.style.Element;
import ej.style.Selector;

/**
 * An odd child selector selects by checking if an element is at an odd position in its parent (0, 2, 4,…).
 * <p>
 * Equivalent to <code>:nth-child(odd)</code> selector in CSS. Its specificity is (0,0,1,0).
 *
 * @see Element#getParentElement()
 * @see SpecificityHelper
 */
public class OddChildSelector implements Selector {

	/**
	 * Odd child selector singleton to avoid creating several ones.
	 */
	public static final OddChildSelector ODD_CHILD_SELECTOR = new OddChildSelector();

	/**
	 * Creates an odd child selector.
	 */
	public OddChildSelector() {
	}

	@Override
	public boolean fit(Element element) {
		Element parentElement = element.getParentElement();

		if (parentElement != null) {
			// Search position in parent.
			Element[] childrenElements = parentElement.getChildrenElements();
			int childrenElementsLength = childrenElements.length;
			for (int i = -1; ++i < childrenElementsLength;) {
				Element elementCandidate = childrenElements[i];
				if (element == elementCandidate) {
					// /!\ Beware, the first index of an array is 0 (even became odd).
					return (i & 1) == 0;
				}
			}
		}
		return false;
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(0, 0, 1, 0);
	}

}
