/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic.drawing;

import ej.microui.display.GraphicsContext;
import ej.microui.display.shape.AntiAliasedShapes;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.style.util.StyleHelper;
import ej.widget.basic.Box;

/**
 * Implementation of a box that represents a radio button.
 * <p>
 * The size of a radio box is dependent on the font size.
 * <p>
 * This example shows a simple radio box:
 *
 * <pre>
 * RadioBox radioBox = new RadioBox();
 * radioBox.setChecked(true);
 * </pre>
 *
 * <img src="../doc-files/radiobox-simple.png" alt="Simple radio box.">
 * <p>
 * This example shows a styled radio box:
 *
 * <pre>
 * RadioBox radioBox = new RadioBox();
 * radioBox.setChecked(true);
 *
 * EditableStyle radioBoxStyle = new EditableStyle();
 * radioBoxStyle.setBorder(new SimpleRoundedBorder(Integer.MAX_VALUE, 2));
 * radioBoxStyle.setPadding(new SimpleOutline(4));
 * radioBoxStyle.setForegroundColor(Colors.BLUE);
 * radioBox.mergeStyle(radioBoxStyle);
 * </pre>
 *
 * <img src="../doc-files/radiobox-styled.png" alt="Styled radio box.">
 * <p>
 * This example shows two styled radio box, one checked, one unchecked. The checked one border color changes:
 *
 * <pre>
 * RadioBox radioBox = new RadioBox();
 * radioBox.setChecked(true);
 * RadioBox radioBox2 = new RadioBox();
 *
 * Stylesheet stylesheet = StyleHelper.getStylesheet();
 *
 * EditableStyle radioBoxStyle = new EditableStyle();
 * radioBoxStyle.setBorder(new SimpleRoundedBorder(Integer.MAX_VALUE, 2));
 * radioBoxStyle.setPadding(new SimpleOutline(4));
 * radioBoxStyle.setForegroundColor(Colors.BLUE);
 * stylesheet.addRule(new TypeSelector(RadioBox.class), radioBoxStyle);
 *
 * EditableStyle checkedRadioboxStyle = new EditableStyle();
 * checkedRadioboxStyle.setBorderColor(Colors.BLUE);
 * stylesheet.addRule(new AndCombinator(new TypeSelector(RadioBox.class), new StateSelector(State.Checked)),
 * 		checkedRadioboxStyle);
 * </pre>
 *
 * <img src="../doc-files/radiobox-styled-checked.png" alt="Styled checked radio box.">
 */
public class RadioBox extends Box {

	private static final int ANTIALIASING_THICKNESS = 2;
	private static final int MINIMUM_SIZE = 8;

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle bounds) {
		if (isChecked()) {
			int width = bounds.getWidth();
			int height = bounds.getHeight();

			g.setColor(style.getForegroundColor());
			int size = Math.min(width, height);
			AntiAliasedShapes antiAliasedShapes = AntiAliasedShapes.Singleton;
			antiAliasedShapes.setFade(1);
			antiAliasedShapes.setThickness(ANTIALIASING_THICKNESS);
			antiAliasedShapes.drawCircle(g, ANTIALIASING_THICKNESS, ANTIALIASING_THICKNESS,
					size - (ANTIALIASING_THICKNESS * 2 + 1));
			g.fillCircle(1, 1, size - (2 + 1));
		}
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle availableSize) {
		int size = Math.max(StyleHelper.getFont(style).getHeight() >> 1, MINIMUM_SIZE);
		if ((size & 0x1) == 0) {
			// Drawing an even circle draws an odd circle -> not centered.
			size++;
		}
		return new Rectangle(0, 0, size, size);
	}

}
