/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container.transition;

import ej.microui.display.Display;
import ej.microui.display.GraphicsContext;
import ej.mwt.MWT;
import ej.style.container.Rectangle;
import ej.style.util.StyleHelper;

/**
 * A slide transition container slides the widgets during the transition.
 *
 * @since 2.3.0
 */
public class SlideScreenshotTransitionContainer extends ScreenshotTransitionContainer {

	private final int direction;
	private final boolean overlap;
	private final boolean clip;

	private int previousStep;

	/**
	 * Creates a slide transition manager.
	 * <p>
	 * The given direction is the way the widgets slide when forward. For example, passing {@link MWT#UP}, the new
	 * widget starts below and goes upward.
	 *
	 * @param direction
	 *            one of {@link MWT#LEFT}, {@link MWT#RIGHT}, {@link MWT#TOP}, {@link MWT#BOTTOM}.
	 * @param overlap
	 *            <code>true</code> if the new widget overlaps the previous one, <code>false</code> if the new widget
	 *            pushes the previous one.
	 * @param clip
	 *            <code>true</code> if the new widget does not move, <code>false</code> if the new widget slides.
	 */
	public SlideScreenshotTransitionContainer(int direction, boolean overlap, boolean clip) {
		// Validate given direction.
		switch (direction) {
		case MWT.UP:
		case MWT.DOWN:
		case MWT.LEFT:
		case MWT.RIGHT:
			break;
		default:
			throw new IllegalArgumentException();
		}
		this.direction = direction;
		this.overlap = overlap;
		this.clip = clip;
	}

	private boolean isHorizontal() {
		switch (this.direction) {
		case MWT.UP:
		case MWT.DOWN:
			return false;
		case MWT.LEFT:
		case MWT.RIGHT:
			return true;
		default:
			// Cannot occur.
			throw new IllegalArgumentException();
		}
	}

	@Override
	public void render(GraphicsContext g) {
		if (isAnimating()) {
			Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
			Rectangle contentBounds = StyleHelper.computeContentBounds(bounds, getStyle());
			int contentX = contentBounds.getX();
			int contentY = contentBounds.getY();
			int contentWidth = contentBounds.getWidth();
			int contentHeight = contentBounds.getHeight();
			g.translate(contentX, contentY);
			g.setClip(0, 0, contentWidth, contentHeight);

			int stop = this.stop;
			int step = this.step;
			int previousStep = this.previousStep;

			boolean newIsBelow = newIsBelow();
			int xPreviousDisplay;
			int yPreviousDisplay;
			int xDisplay;
			int yDisplay;
			int xScreenshot;
			int yScreenshot;
			int widthScreenshot;
			int heightScreenshot;
			if (isHorizontal()) {
				xDisplay = step;
				yDisplay = 0;
				xPreviousDisplay = previousStep;
				yPreviousDisplay = 0;
				yScreenshot = 0;
				if (newIsBelow) {
					if (this.direction == MWT.LEFT) {
						xScreenshot = 0;
						widthScreenshot = xDisplay;
					} else {
						xScreenshot = xDisplay - stop;
						widthScreenshot = -xDisplay;
					}
				} else {
					xScreenshot = xDisplay - stop;
					widthScreenshot = contentWidth;
				}
				heightScreenshot = contentHeight;
			} else {
				xDisplay = 0;
				yDisplay = step;
				xPreviousDisplay = 0;
				yPreviousDisplay = previousStep;
				xScreenshot = 0;
				widthScreenshot = contentWidth;
				if (newIsBelow) {
					if (this.direction == MWT.TOP) {
						yScreenshot = 0;
						heightScreenshot = yDisplay;
					} else {
						yScreenshot = yDisplay - stop;
						heightScreenshot = -yDisplay;
					}
				} else {
					yScreenshot = yDisplay - stop;
					heightScreenshot = contentHeight;
				}
			}

			// Draw display on itself.
			if (!this.forward ? !this.clip : !this.overlap) {
				drawDisplay(g, contentWidth, contentHeight, xPreviousDisplay, yPreviousDisplay, xDisplay, yDisplay,
						step - previousStep);
			}

			// Draw new widget screenshot.
			if (newIsBelow) {
				g.clipRect(xScreenshot, yScreenshot, widthScreenshot, heightScreenshot);
			} else if (this.clip && this.forward) {
				g.clipRect(xScreenshot, yScreenshot, widthScreenshot, heightScreenshot);
			} else {
				g.translate(xScreenshot, yScreenshot);
				g.clipRect(0, 0, contentWidth, contentHeight);
			}
			g.drawImage(this.newScreenshot, 0, 0, GraphicsContext.LEFT | GraphicsContext.TOP);

			this.previousStep = step;
		} else {
			super.render(g);
		}
	}

	// FIXME Support should be done by MicroUI stack (done since version 2.2.0).
	private void drawDisplay(GraphicsContext g, int width, int height, int xPreviousDisplay, int yPreviousDisplay,
			int xDisplay, int yDisplay, int move) {
		Display display = Display.getDefaultDisplay();
		int absoluteX = g.getTranslateX();
		int absoluteY = g.getTranslateY();
		if (move <= 0) {
			g.drawRegion(display, absoluteX + xPreviousDisplay, absoluteY + yPreviousDisplay, width, height, xDisplay,
					yDisplay, GraphicsContext.LEFT | GraphicsContext.TOP);
		} else {
			if (isHorizontal()) {
				int currentX = width - move;
				while (currentX - xPreviousDisplay + move > 0) {
					g.drawRegion(display, absoluteX + currentX, absoluteY + 0, move, height, currentX + move, 0,
							GraphicsContext.LEFT | GraphicsContext.TOP);
					currentX -= move;
				}
			} else {
				int currentY = height - move;
				while (currentY - yPreviousDisplay + move > 0) {
					g.drawRegion(display, absoluteX, absoluteY + currentY, width, move, 0, currentY + move,
							GraphicsContext.LEFT | GraphicsContext.TOP);
					currentY -= move;
				}
			}
		}
	}

	@Override
	protected int getStop(int contentWidth, int contentHeight) {
		switch (getActualDirection(this.forward)) {
		case MWT.UP:
			return -contentHeight;
		case MWT.DOWN:
			return contentHeight;
		case MWT.LEFT:
			return -contentWidth;
		case MWT.RIGHT:
			return contentWidth;
		default:
			// Cannot occur.
			throw new IllegalArgumentException();
		}
	}

	@Override
	protected void resetContext() {
		super.resetContext();
		this.previousStep = 0;
	}

	// The new one is below if overlap and not forward.
	private boolean newIsBelow() {
		return this.overlap && !this.forward;
	}

	private int getActualDirection(boolean forward) {
		if (forward) {
			return this.direction;
		} else {
			switch (this.direction) {
			case MWT.UP:
				return MWT.DOWN;
			case MWT.DOWN:
				return MWT.UP;
			case MWT.LEFT:
				return MWT.RIGHT;
			case MWT.RIGHT:
				return MWT.LEFT;
			default:
				// Cannot occur.
				throw new IllegalArgumentException();
			}
		}
	}

}
