/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package googlecloud.iotcore;

import googlecloud.iotcore.pubsub.Client;
import googlecloud.iotcore.pubsub.ConnectOptions;
import googlecloud.iotcore.pubsub.IotCoreException;
import googlecloud.iotcore.pubsub.TopicListener;

/**
 * This class provides a Device reification that matches the device in Google Cloud IoT Core. It communicates
 * automatically on known resources attached to the device to report states, events and get back the configuration of
 * the device.
 */
public class Device {

	private static final String DEVICE_NOT_CONNECTED = "Device not connected.";
	private static final String DEVICE_ALREADY_CONNECTED = "Device already connected.";

	private static final String STATE_TOPIC = "state";
	private static final String CONFIG_TOPIC = "config";
	private static final String EVENTS_TOPIC = "events";

	private final Client client;

	private final ConnectOptions options;

	private ConfigListener listener;

	private boolean connected;

	/**
	 * Create a Device with connection information related to the Google Cloud IoT Core services.
	 *
	 * @param options
	 *            the options of connection to the broker
	 */
	public Device(ConnectOptions options) {
		this.options = options;
		this.client = new Client();
	}

	/**
	 * Gets the JSON Web Token once connected.
	 *
	 * @return the jwt once connected, null otherwise.
	 */
	public char[] getJwt() {
		return this.client.getJwt();
	}

	/**
	 * Sets the JSON Web Token to be used for connection.
	 *
	 * @param jwt
	 *            the jwt to used for connection.
	 */
	public void setJwt(char[] jwt) {
		this.client.setJwt(jwt);
	}

	/**
	 * Set the state of Device.
	 *
	 * @param data
	 *            application-dependent state data
	 * @throws IotCoreException
	 *             on connection issues and errors of communication
	 */
	public void setState(byte[] data) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException(DEVICE_NOT_CONNECTED);
		}

		this.client.publish(this.options.buildTopic(STATE_TOPIC), data);
	}

	/**
	 * Set events.
	 *
	 * @param data
	 *            application-dependent event data
	 * @throws IotCoreException
	 *             on connection issues and errors of communication
	 */
	public void setEvent(byte[] data) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException(DEVICE_NOT_CONNECTED);
		}

		this.client.publish(this.options.buildTopic(EVENTS_TOPIC), data);
	}

	/**
	 * Set sub-events.
	 *
	 * @param subEvent
	 *            the wanted sub-folder of events
	 * @param data
	 *            application-dependent sub-event data
	 * @throws IotCoreException
	 *             on connection issues and errors of communication
	 */
	public void setSubEvent(String subEvent, byte[] data) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException(DEVICE_NOT_CONNECTED);
		}

		this.client.publish(this.options.buildTopic(EVENTS_TOPIC + "/" + subEvent), data);
	}

	/**
	 * Add a configuration listener notified when the config of the device changes and at first connection.
	 *
	 * @param listener
	 *            the right ConfigListener that will get notified
	 * @throws IotCoreException
	 *             on subscription error to the config topic
	 *
	 */
	public void setConfigListener(ConfigListener listener) throws IotCoreException {

		if (this.listener == null) {

			this.listener = listener;

			this.client.setTopicListener(new TopicListener() {

				@Override
				public void onDataReceived(String topic, byte[] dataReceived) {

					ConfigListener listener = Device.this.listener;

					if (listener != null && topic.equals(Device.this.options.buildTopic(CONFIG_TOPIC))) {
						Device.this.listener.onConfigChanged(dataReceived);
					}

				}
			});

			this.client.subscribe(Device.this.options.buildTopic(CONFIG_TOPIC));
		}

	}

	/**
	 * Stop processing config events.
	 */
	public void clearConfigListener() {
		this.listener = null;
	}

	/**
	 * Connect the Device to the Google Cloud IoT Core services.
	 *
	 * @throws IotCoreException
	 *             when connected
	 */
	public void connect() throws IotCoreException {

		if (isConnected()) {
			throw new IotCoreException(DEVICE_ALREADY_CONNECTED);
		}

		this.client.connect(this.options);

		this.connected = true;
	}

	/**
	 * Disconnect the Device from the Google Cloud IoT Core services.
	 *
	 * @throws IotCoreException
	 *             when not connected
	 */
	public void disconnect() throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException(DEVICE_NOT_CONNECTED);
		}

		this.client.disconnect();

		this.connected = false;
	}

	/**
	 * Check if the device is connected.
	 *
	 * @return the connection state, true connected, false not connected
	 */
	public boolean isConnected() {
		this.connected = this.client.isConnected();
		return this.connected;
	}

}
